"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.robustPolynomial = robustPolynomial;
const calcCoefficients_1 = require("./calcCoefficients");
const getRandomTuples_1 = require("./getRandomTuples");
const predict_1 = require("./predict");
const residualsMedian_1 = require("./residualsMedian");
function robustPolynomial(regression, x, y, degree) {
    const powers = new Array(degree).fill(0).map((_, index) => index);
    const tuples = (0, getRandomTuples_1.getRandomTuples)(x, y, degree);
    let min;
    for (const tuple of tuples) {
        const coefficients = (0, calcCoefficients_1.calcCoefficients)(tuple, powers);
        const residuals = [];
        for (let j = 0; j < x.length; j++) {
            const residual = y[j] - (0, predict_1.predict)(x[j], powers, coefficients);
            residuals[j] = {
                residual: residual * residual,
                coefficients,
            };
        }
        const median = (0, residualsMedian_1.residualsMedian)(residuals);
        if (!min || median.residual < min.residual) {
            min = median;
        }
    }
    regression.degree = degree;
    regression.powers = powers;
    regression.coefficients = min ? min.coefficients : [];
}
//# sourceMappingURL=robustPolynomial.js.map